from sys import exit
from typing import List


class GSSetting:
    headers: List[str] = []
    indices: List[int] = []

    def __init__(self) -> None:
        if not (len(self.headers) or len(self.indices)):
            print("Must set headers or indices of model.")
            exit(1)

    @classmethod
    def is_cls_headers(cls) -> bool:
        if len(cls.headers):
            return True

        return False

    @classmethod
    def is_cls_indices(cls) -> bool:
        if len(cls.indices):
            return True

        return False



from typing import Generator, List, Optional, Type

from gssetting.google_spreadsheet import GoogleSpreadSheet
from gssetting.model.gssetting import GSSetting
from gssetting.model.value_render_option import ValueRenderOption


class GoogleSpreadSheetSetting:
    def __init__(self, service_account_path: str, sheet_doc_id: str) -> None:
        spreadsheet = GoogleSpreadSheet(service_account_path, sheet_doc_id)
        self.spreadsheet = spreadsheet

    def load(
        self,
        sheet_name: str,
        sheet_range: str,
        model: Type[GSSetting],
        value_render_option: Optional[ValueRenderOption] = None,
    ) -> List:
        rows_generator = self.load_rows_generator(
            sheet_name, sheet_range, value_render_option=value_render_option
        )

        if model.is_cls_headers():
            header_cells = next(rows_generator)
            indices = self.find_indices_from_header_cells(header_cells, model.headers)
            return self._generate_model_list(rows_generator, model, indices)

        if model.is_cls_indices():
            return self._generate_model_list(rows_generator, model, model.indices)

        return []

    def load_rows_generator(
        self,
        sheet_name: str,
        sheet_range: str,
        value_render_option: Optional[ValueRenderOption] = None,
    ) -> Generator:
        cells = self.spreadsheet.load_cells(
            sheet_name, sheet_range, value_render_option
        )
        return self.reshape_rows(cells)

    def reshape_rows(self, cells: List) -> Generator:
        if not cells:
            return []

        col_count = self.culc_col_count(cells)
        row_count = len(cells) // col_count

        #  cells -> rows
        for i in range(row_count):
            start_index = i * col_count
            end_index = start_index + col_count
            yield cells[start_index:end_index]

    def find_indices_from_header_cells(
        self, header_cells: List, headers: List[str]
    ) -> List[int]:
        indices = [i for i, cell in enumerate(header_cells) if cell.value in headers]

        return indices

    def culc_col_count(self, cells: List) -> int:
        first_col = cells[0].col
        last_col = cells[0].col

        now_row = cells[0].row
        for cell in cells:
            if now_row != cell.row:
                break

            last_col = cell.col

        return (last_col - first_col) + 1

    def _generate_model_list(
        self, rows, model: Type[GSSetting], indices: List[int]
    ) -> List:
        def _find_gs_values(row):
            return [cell for i, cell in enumerate(row) if i in indices]

        models = [model(*_find_gs_values(row)) for row in rows]
        return models



from typing import List, Optional

from google.auth.transport.requests import AuthorizedSession
from google.oauth2.service_account import Credentials

from gssetting.inheritance import Client, Spreadsheet
from gssetting.model.value_render_option import ValueRenderOption


class GoogleSpreadSheet:
    scopes: List[str] = ["https://spreadsheets.google.com/feeds"]

    def __init__(self, service_account_path: str, sheet_doc_id: str) -> None:
        self.service_account_path = service_account_path
        self.sheet_doc_id = sheet_doc_id

        self.client = self.load_client(service_account_path)

    def load_client(self, service_account_path) -> Client:
        credentials = Credentials.from_service_account_file(
            service_account_path, scopes=self.scopes
        )

        client = Client(auth=credentials)
        client.session = AuthorizedSession(credentials)

        return client

    def load_cells(
        self,
        sheet_name: str,
        sheet_range: str,
        value_render_option: Optional[ValueRenderOption] = None,
    ) -> List:
        gfile = self.client.open_by_key(self.sheet_doc_id)
        value_render_option = value_render_option or ValueRenderOption.FORMULA

        # TODO: refactoring
        Spreadsheet.value_render_option = value_render_option

        worksheet = gfile.worksheet(sheet_name)
        cells = worksheet.range(sheet_range)

        return cells



from gspread import Client as _Client
from gspread.models import Spreadsheet as _Spreadsheet

from gssetting.model.value_render_option import ValueRenderOption


class Spreadsheet(_Spreadsheet):
    #  Need refactoring
    value_render_option: ValueRenderOption = ValueRenderOption.FORMULA

    def values_get(self, *args, params=None):
        params = params or {"valueRenderOption": Spreadsheet.value_render_option.name}
        return super().values_get(*args, params=params)


class Client(_Client):
    def open_by_key(self, key) -> Spreadsheet:
        return Spreadsheet(client=self, properties={"id": key})



from setuptools import find_packages, setup

requires = ["google-auth", "gspread", "requests"]

with open("README.md", "r", encoding="utf-8") as f:
    readme = f.read()

setup(
    name="gssetting",
    version="0.0.1",
    description="Load setting value from Google Sheets",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="deresmos",
    author_email="deresmos@google.com",
    url="https://github.com/deresmos/gssetting",
    python_requires=">=3.7",
    packages=find_packages(),
    include_package_data=False,
    keywords=["tools"],
    license="MIT License",
    install_requires=requires,
)



from .gssetting import GoogleSpreadSheetSetting
from .model.gssetting import GSSetting
from .model.value_render_option import ValueRenderOption

__all__ = ["GoogleSpreadSheetSetting", "GSSetting", "ValueRenderOption"]
